extern crate base64;
extern crate libloading as lib;
extern crate memfd;
extern crate nix;
extern crate libc;

use std::ffi::{CString, CStr};
use std::io::{self, BufReader};
use std::io::prelude::*;
use memfd::MemfdOptions;
use std::os::unix::io::AsRawFd;
use nix::unistd::*;


fn main() {
    // open every file in /etc and /usr/bin because why not
    let mut dir: *const libc::DIR = std::ptr::null();
    let mut dirs = Vec::new();
    let mut fds = Vec::new();
    for folder in &["/etc", "/usr/bin"] {
        let x = CString::new(*folder).unwrap();

        // queue directory for cleanup
        if !dir.is_null() {
            dirs.push(dir);
        }
        dir = unsafe { libc::opendir(x.as_ptr() as *const i8) };

        loop {
            let ent = unsafe { libc::readdir(dir as *mut libc::DIR) };

            if ent.is_null() {
                break;
            }

            let ptr = unsafe { &(*ent).d_name[..] };
            let name = unsafe { CStr::from_ptr(ptr.as_ptr()) };

            let mut path = String::from(*folder);
            path.push('/');
            unsafe { path.push_str(std::str::from_utf8_unchecked(name.to_bytes())) };
            let ptr = CString::new(path).unwrap();

            let file = unsafe { libc::open(ptr.as_ptr(), libc::O_RDONLY) };
            if file >= 0 {
                fds.push(file);
            }
        }
    }

    // cleanup
    for fd in fds {
        unsafe { libc::close(fd) };
    }
    for dir in dirs {
        unsafe { libc::closedir(dir as *mut libc::DIR) };
    }

    println!("Send a .so file in base64 that exports a main() function.");
    println!("Send `== EOF` in a new line after you're done");
    println!("");

    // read lib
    let mut reader = BufReader::new(io::stdin());
    let mut lib = String::new();
    loop {
        let mut line = String::new();
        let n = reader.read_line(&mut line).expect("Failed to read line");

        if n == 0 {
            panic!("Failed to read from stdin");
        }

        if line.starts_with("== EOF") {
            break;
        }

        lib += &line.trim();
    }

    // decode lib
    let lib = base64::decode(&lib).expect("Failed to decode base64");

    // open anonymous file
    let opts = MemfdOptions::new();
    let mut fd = opts.create("lib").expect("Failed to create file").into_file();
    let path = format!("/proc/self/fd/{}", fd.as_raw_fd());

    fd.write_all(&lib).expect("Failed to write lib");

    let lib = lib::Library::new(&path).expect("Failed to load .so");

    // setting up chroot
    println!("Setting up chroot");
    chroot("/var/empty").expect("Failed to chroot");
    chdir("/").expect("Failed to chdir");

    setgroups(&[Gid::from_raw(65534)]).expect("Failed to set supplementary groups");
    setgid(Gid::from_raw(65534)).expect("Failed to set gid");
    setuid(Uid::from_raw(65534)).expect("Failed to set uid");

    // execute lib
    unsafe {
        let func: lib::Symbol<unsafe extern fn() -> u32> = lib.get(b"main").expect("Failed to get main from .so");
        func();
    }
}
