require 'sinatra'
require 'faccade'


class App < Sinatra::Base

  enable :sessions

  helpers do

    def authenticated_user
      session[:user_id]
    end

    def logged_in?
      !authenticated_user.nil?
    end

    def failed_login?
      failed_login = session[:failed_login]
      session[:failed_login] = nil
      failed_login
    end

    def logout
      session[:user_id] = nil
    end

    def identification_error?
      failed_identification = session[:failed_identification]
      session[:failed_identification] = nil
      failed_identification
    end

  end

  def logic
    settings.faccade
  end

  configure do
    set :faccade, Faccade.new
    set :environment, :production
  end

  not_found do
    status 404
    erb :notfound
  end

  get '/' do
    erb :login_username
  end

  get '/login_username' do
    erb :login_username
  end

  post '/login' do
    identification = params['identification'] || ""
    password       = params['password']
    if logic.user_exists?(identification)
      if password
        user = logic.login(identification, password)
        if user
          session[:user_id] = user
          erb :home
        else
          session[:failed_login] = true
          status 400
          erb :login_password, locals: { identification: identification }
        end
      else
        erb :login_password, locals: { identification: identification }
      end
    else
      session[:failed_identification] = true
      status 400
      erb :login_username
    end
  end

  get '/login' do
    erb :login_username
  end

  get '/logout' do
    logout if logged_in?
    erb :login_username
  end

end
